SystemOrganization addCategory: #Sensors!

SortedCollection variableSubclass: #SortedSequence
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!SortedSequence commentStamp: '<historical>' prior: 0!
I add one method to SortedCollection.  It would be just as easy to add the method
directly to the class instead of making a subclass, but this makes it easier to see
the new method.

The method is indexOfStartOfIntervalContaining: anElement
It returns the index of anElement or, if it is not present, the index of the
largest element smaller than it.  It is useful when you want to have a
table that is sorted by keys and you want to look up in the table to
find the nearest match to a key.!

!SortedSequence methodsFor: 'accessing' stamp: 'WeiqiLuo 7/6/2013 10:52'!
indexOfStartOfIntervalContaining: anElement 
	"Return the index of anElement or, if it is not present, of the index of the
	largest element smaller than it."

	self isEmpty ifTrue: [^0].
	^(self indexForInserting: anElement) - firstIndex! !

WARestfulHandler subclass: #DeviceWS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 22:18'!
addDevice
	|xml device|
    <post>
    <path: 'Device'>
	xml := (self requestContext request) rawBody.
	Transcript show: xml.
	device := Device newFromXml: xml.
	DeviceManager singleton addDevice: device.
	^'<Device>', (device name) , '</Device>'.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 21:25'!
getDevice: deviceName
    <get>
    <path: 'Device/{1}'>
    <produces: 'text/xml'>
	^(DeviceManager singleton getDevice: deviceName) asXml.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 21:25'!
getDevices
    <get>
    <path: 'Device'>
    <produces: 'text/xml'>
	|stream|
	stream := WriteStream on: (String new: 1).
	stream nextPutAll: '<Devices>'.
	DeviceManager singleton devices valuesDo: [:device | 
			stream nextPutAll: device asXml.
		].
	stream nextPutAll: '</Devices>'.
	^stream contents.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/2/2013 21:02'!
postData: sensor
	|xml device|
    <post>
    <path: 'Data/{1}'>
	xml := (self requestContext request) rawBody.
	Transcript show: xml.
	^'<Data></Data>'.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 21:33'!
postData: aDeviceName sensorName: aSensorName 
	|xml device doc stream data sensor|
    <post>
    <path: 'Data/{1}/{2}'>
	xml := (self requestContext request) rawBody.
	Transcript show: xml.
	stream := xml readStream.
	doc := XMLDOMParser parseDocumentFrom: stream.
	data := (doc findElementNamed: 'data') contentString asNumber.
	
	sensor := (DeviceManager getDevice: aDeviceName) getSensorByName: aSensorName.
	
	sensor updateDataNow: data.
	
	^'Received <data>', data asString ,'</data> for: ', aDeviceName, ' ', aSensorName.! !

!DeviceWS methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 21:25'!
removeDevice: deviceName
    <delete>
    <path: 'Device/{1}'>
    <produces: 'text/xml'>
   	DeviceManager singleton removeDevice: deviceName.
	^'<Device>', deviceName , '</Device>'.! !

WAComponent subclass: #WebChartFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebChartFactory class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/12/2013 02:16'!
createChart: aDeviceSensor

|xdata ydata|
(aDeviceSensor type = #LightSensor) ifTrue: [
	Transcript show: 'this is a light sensor'.
	xdata := (aDeviceSensor data) second collect: [:sec | DateAndTime fromSeconds: sec].
	ydata := aDeviceSensor data value.
	^ WebLineChart new xData: xdata yData: ydata title: 'Light Data' units: 'Lumens'].

(aDeviceSensor type = #TemperatureSensor) ifTrue: [
	Transcript show: 'this is a temperature sensor'.
	xdata := (aDeviceSensor data) second collect: [:sec | DateAndTime fromSeconds: sec].
	ydata := aDeviceSensor data value.
	^ WebLineChart new xData: xdata yData: ydata title: 'Temperature Data' units: 'Degrees Fahrenheit'].

^nil.
"createSensor: aString
	aString = 'LightSensor' ifTrue: [^ LightSensor new].
	aString = 'TemperatureSensor' ifTrue: [^ TemperatureSensor new].
	^nil"! !

WAComponent subclass: #WebComponent
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

WebComponent subclass: #WebAlarmView
	instanceVariableNames: 'lowerThreshold upperThreshold name device deviceSensor alarmName deviceName alarm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:50'!
alarm: anAlarm

alarm := anAlarm

! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 00:52'!
deviceSensorList
^deviceSensorList ! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 00:52'!
getDevicesForList
	| deviceList deviceSelectList |
	deviceSelectList := OrderedCollection new.
	deviceList := DeviceManager singleton devices.
	deviceList
		keysAndValuesDo: [ :device :deviceObject | deviceObject sensors keysAndValuesDo: [ :sensor :sensorObject | deviceSelectList add: device , '~' , sensor ] ].
	deviceSensorList := deviceSelectList! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 20:31'!
renderAlarmLowerOn: html

	html
		listItem: [ 
			self
				renderFormElementOn: html
				element: [ 
					html label
						for: html nextId;
						with: 'Lower Threshold:'.
					html textInput
						id: html lastId;
						value: alarm lower ;
						callback: [ :val | lowerThreshold := val ] ] ]! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 20:10'!
renderAlarmNameTextOn: html
	html
		listItem: [ 
			self
				renderFormElementOn: html
				element: [ 
					html label
						for: html nextId;
						with: 'Alarm Name:'.
					html textInput
						id: html lastId;
						value: (alarm name );
						callback: [ :val | alarmName := val ] ] ]! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 21:40'!
renderAlarmOn: html

	alarm
		ifNil: [ 
			alarm := Alarm
				setAlarm: 'New Alarm'
				device: Device new
				deviceSensor: LightSensor new
				lower: 0
				upper: 0 ].

	html
		form: [ 
			html unorderedList
				jqmListView;
				jqmEnableInset;
				with: [ 
							self renderAlarmNameTextOn: html.
							html listItem: [ self renderSelectDevicesOn: html ].
							self renderAlarmLowerOn: html.
							self renderAlarmUpperOn: html.
							html listItem with: [ 
								html fieldSet
									jqmContainerTwoColumn;
									with: [ 
										html div
											jqmColumnOne;
											with: [ 
												html submitButton
													callback: [ self answer ];
													with: 'Cancel' ].
										html div
											jqmColumnTwo;
											with: [ 
												html submitButton
													jqmTheme: 'a';
													callback: [ self save ];
													with: 'Submit' ] 
											] 
										] 
									] 
								]! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 20:13'!
renderAlarmUpperOn: html
	html
		listItem: [ 
			self
				renderFormElementOn: html
				element: [ 
					html label
						for: html nextId;
						with: 'Upper Threshold:'.
					html textInput
						id: html lastId;
						value: alarm upper ;
						callback: [ :val | upperThreshold := val ] ] ]! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 12:02'!
renderContentOn: html

self renderDefaultHeaderOn: html.
html div
      jqmContent;
      with: [
         html paragraph: '' .
 html heading: 'Create/Update Alarm' level: 2.
self renderAlarmOn: html.].	

self renderDefaultFooterOn: html.! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 00:59'!
renderFormElementOn: html element: elementBlock
	html div
		jqmFieldContainer;

		with: [
			elementBlock value ]! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 21:11'!
renderSelectDevicesOn: html

|selectedDeviceSensor|
selectedDeviceSensor := (alarm device name), '~', (alarm deviceSensor name).
Transcript show: selectedDeviceSensor.

	self getDevicesForList.
	html
		form: [ 
			self
				renderFormElementOn: html
				element: [ 
					html label
						for: html nextId;
						with: 'Device-Sensor Observed:'.
					html select
						id: html lastId;
						selected: selectedDeviceSensor;
						callback: [ :val | deviceSensor := val ];
						list: self deviceSensorList.] ]! !

!WebAlarmView methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 21:37'!
save
	| curDevice curDeviceName curDeviceSensor alarmNew deviceTemp |
	curDevice := deviceSensor subStrings: '~'.
	curDeviceName := curDevice first.
	curDeviceSensor := curDevice last.
	
	device := DeviceManager singleton getDevice: curDeviceName.
	deviceSensor := device getSensorByName: curDeviceSensor.
	
	alarmNew := (Alarm new)
		setAlarm: alarmName
		device: device
		deviceSensor: deviceSensor
		lower: lowerThreshold
		upper: upperThreshold.

		
	AlarmManager singleton addAlarm: alarmNew.

	self answer.! !

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 01:56'!
children

^ Array with: header! !

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 16:04'!
initialize

super initialize.
header := WebHeader new! !

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/8/2013 23:46'!
renderDefaultFooterOn: html

html div
      jqmFooter;
      jqmFixedPosition;
      jqmId: 'fixedFooter';
      with: [
         html div
            jqmNavBar;
            with: [
               html unorderedList: [
                  html listItem: [
                     html anchor
                        jqmTransitionFade;
                        callback: [ self decoratedComponent show: (WebLights new) ];	
                        with: 'Lights' ].
                  html listItem: [
                     html anchor
                        jqmTransitionFade;
                        callback: [ self decoratedComponent show: (WebTemperature new) ];	
                        with: 'Temperature' ].
                  html listItem: [
                     html anchor
                        jqmTransitionFade;
                        callback: [ self decoratedComponent show: (WebReports  new) ];
                        with: 'Reports' ].
                html listItem: [
                     html anchor
                        jqmTransitionFade;
                        callback: [ self decoratedComponent show: (WebConfiguration   new) ];
                        with: 'Configuration' ].
                   ]
		]]! !

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 00:22'!
renderDefaultHeaderOn: html
html div
		jqmHeader;
		jqmTheme: 'b';
		with: [
			html heading
				level1;
				with: self class name.
				
			html anchor
				jqmIcon: 'home';
				jqmReverse;
				jqmTransitionSlide;
				jqmIconNoText;
				callback: [ 
					self session presenter home ];
				with: 'home' ]! !

!WebComponent methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/11/2013 22:45'!
renderFormElementOn: html element: elementBlock
	html div
		jqmFieldContainer;

		with: [
			elementBlock value ]! !

WebComponent subclass: #WebConfiguration
	instanceVariableNames: 'selectedCategory selectedDevice deviceSensorList alarms'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:29'!
alarms

^ AlarmManager singleton alarms
! !

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 11:13'!
renderAlarmListContentsOn: html
	
	self alarms do: [ :each | 
		html listItem: [
			html anchor
				callback: [self call: ((WebAlarmView new) alarm: each)];
				with: [
					html text: each name]].
			]
		! !

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 09:58'!
renderAlarmListOn: html
	 html unorderedList 
		jqmListView;
		jqmEnableInset;
		jqmTheme: 'd';
		class: 'alarms';
		with: [
			html listItem
				jqmListDivider;
				with: 'Alarms'.
			self renderAlarmListContentsOn: html ]

! !

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 09:59'!
renderContentOn: html

self renderDefaultHeaderOn: html.

html div
      jqmContent;
      with: [
		html anchor
        		callback: [ self call: (WebAlarmView new) ];
        		with: 'Add Alarm'.
		html break.
		self renderAlarmListOn: html.
		].	


self renderDefaultFooterOn: html! !

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 11:06'!
renderNavigationMenu: html
	html
		anchor
				callback: [ self returnHome ];
				with: 'Return Home' ! !

!WebConfiguration methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 11:06'!
returnHome

	self answer! !

WebComponent subclass: #WebHome
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebHome class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 00:37'!
canBeRoot

^ true! !

!WebHome class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 17:13'!
initialize

	| app |

	app := WAAdmin register: self asApplicationAt: 'sensorHome'.

	app

		addLibrary: JQDevelopmentLibrary;

		addLibrary: HCDevelopmentLibrary.

	app preferenceAt: #scriptGeneratorClass put: JQScriptGenerator.! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/9/2013 23:59'!
renderAlarmListContentsOn: html
	html listItem: [
		html anchor: [
			html text: 'Alarm1 - Sensor 1 temperature exceeded 75 degrees fahrenheit ']].
	html listItem: [
		html anchor: [
			html text: 'Alarm2 - Sensor 2 luminosity exceeded 25'.]].
! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 00:01'!
renderAlarmListOn: html
	 html unorderedList 
		jqmListView;
		jqmEnableInset;
		jqmTheme: 'd';
		class: 'alarms';
		with: [
			html listItem
				jqmListDivider;
				with: 'Alarms'.
			self renderAlarmListContentsOn: html ]

! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:16'!
renderContentOn: html

self renderDefaultHeaderOn: html.
html div
      jqmContent;
      with: [
         html paragraph: '' ].	

self renderAlarmListOn: html.
html break.
self renderScheduleListOn: html.
html break.
self renderNotificationListOn: html.
self renderDefaultFooterOn: html! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:19'!
renderNotificationListContentsOn: html
	|messages|
	
	messages := NotificationManager singleton messages.
	messages size == 0 ifTrue: [
	html listItem: [
		html anchor: [
			html text: 'No message available.']].
	] ifFalse: [
		messages do: [:message | 
			html listItem: [
			html anchor: [
				html text: message]].
	
			]
	].
! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:17'!
renderNotificationListOn: html
	 html unorderedList 
		jqmListView;
		jqmEnableInset;
		jqmTheme: 'd';
		class: 'alarms';
		with: [
			html listItem
				jqmListDivider;
				with: 'Notifications'.
			self renderNotificationListContentsOn: html ]

! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 00:03'!
renderScheduleListContentsOn: html
	html listItem: [
		html anchor: [
			html text: 'Activating Control 1 at 15:00']].
	html listItem: [
		html anchor: [
			html text: 'Activating Control 2 at 17:00'.]].
! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 00:02'!
renderScheduleListOn: html
	 html unorderedList 
		jqmListView;
		jqmEnableInset;
		jqmTheme: 'd';
		class: 'alarms';
		with: [
			html listItem
				jqmListDivider;
				with: 'Schedule'.
			self renderScheduleListContentsOn: html ]

! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 19:49'!
style
	^ '
		.alarms {
			width: 80%;
			margin: auto;
			font-family: verdana;
			color: #3e3e3e;
			font-size:13px;
		}'
		
! !

!WebHome methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 19:54'!
updateRoot: aRoot
    super updateRoot: aRoot.
    aRoot javascript url: '/yourServer/someJsLibrary.js'.
    aRoot stylesheet url: '/yourServer/someCSSLibrary.css'! !

WebComponent subclass: #WebLights
	instanceVariableNames: 'radio testValue'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebLights methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/18/2013 22:27'!
radio
	^ radio ifNil: [ radio := #On ]! !

!WebLights methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/18/2013 22:27'!
radio: anObject
	radio := anObject! !

!WebLights methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/19/2013 01:53'!
renderContentOn: html

self renderDefaultHeaderOn: html.
html div
      jqmContent;
      with: [
         html paragraph: ''.

DeviceManager singleton devices valuesDo: [:device | 
	device sensors valuesDo: [:sensor | 
		(sensor isKindOf: Control) ifTrue: [
				html form 
					with: [
					html div class:'label'; with: (device name), '-', (sensor name).
					self renderRadioButtonOn: html forDevice: device sensor: sensor]. 
					].
			]
		].

].


self renderDefaultFooterOn: html! !

!WebLights methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 18:08'!
renderFlipSwitchOn: html

   self renderFormElementOn: html element: [

      html select
         list: #(On Off);
         jqmSlider;
		jqmTheme: 'b'].! !

!WebLights methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 18:08'!
renderFlipSwitchOn: html label: name

   self renderFormElementOn: html element: [

	html text: name.
	
      html select
         list: #(On Off);
         jqmSlider;
		jqmTheme: 'b'].! !

!WebLights methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/19/2013 01:56'!
renderRadioButtonOn: html forDevice: device sensor: sensor
	| radioLabel |
	radioLabel := html nextId.
	
	html fieldSet
		jqmControlGroup;
		jqmHorizontal;
		with: [
			html radioGroup
				selected: self radio; 
				callback: [ :value | self radio: value ];
				with: [ :group |
					#(On Off) do: [ :each | 
						html label 
							for: html nextId;
							with: each.
						html radioButton
							group: group;
							id: html lastId;
							onChange:  (html jQuery ajax serializeForm callback: [
								Transcript show: radio. 
								radio = 'Off' ifTrue:[sensor sendToDevice: 0] ifFalse:[sensor sendToDevice: 1].
								]);
							value: each ] .
						 .] .
				]
	
! !

!WebLights methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 08:02'!
style
	^ '
		.alarms {
			width: 80%;
			margin: auto;
			font-family: verdana;
			color: #3e3e3e;
			font-size:13px;
		}'
		! !

WebComponent subclass: #WebReports
	instanceVariableNames: 'selectedDevice deviceSensorList'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/12/2013 00:22'!
deviceSensorList
^deviceSensorList ! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 00:47'!
getDevicesForList
	| deviceList deviceSelectList |
	deviceSelectList := OrderedCollection new.
	deviceList := DeviceManager singleton devices.
	deviceList
		keysAndValuesDo: [ :device :deviceObject | deviceObject sensors keysAndValuesDo: [ :sensor :sensorObject | deviceSelectList add: device , '~' , sensor ] ].
	deviceSensorList := deviceSelectList! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 16:41'!
initialize 

super initialize.
! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 17:01'!
renderBarChartOn: html
	html div
		script:
			(html
				highchart: [ :chart | 
					chart chart defaultSeriesType: 'pie'.
					chart title text: 'Light On/Off'.
					chart series: (Array with: (chart step data: #(#('On' 80) #(#Off 20)))) ])! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/12/2013 01:43'!
renderContentOn: html
	| currentDeviceSensor currentDeviceName currentSensorName currentDevice currentSensor |
	self renderDefaultHeaderOn: html.
	html div
		jqmContent;
		with: [ 
					self renderSelectDevicesOn: html.
					html text: 'Selected Device: '.
					selectedDevice isNil
						ifFalse: [ 
							html text: selectedDevice.
							
							currentDeviceSensor := selectedDevice subStrings: '~'.
							currentDeviceName := currentDeviceSensor first.
							currentSensorName := currentDeviceSensor last.
							
							currentDevice := DeviceManager singleton getDevice: currentDeviceName.
							currentSensor := currentDevice getSensorByName: currentSensorName.
							
							
							html render: (WebChartFactory createChart: currentSensor) ] ].
	self renderDefaultFooterOn: html! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 12:08'!
renderSelectDevicesOn: html
	self getDevicesForList.
	html form 
		with:[ 
			self renderFormElementOn: html
				element: [ 
					html select
						id: html lastId;
						selected: selectedDevice;
						callback: [ :val | selectedDevice := val ];
						list: self deviceSensorList.
			self renderFormElementOn: html element: [
				html submitButton
					jqmInline ;
						jqmTheme: 'a';
						with: 'Submit' ] ]]! !

!WebReports methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/11/2013 23:00'!
renderSubmitCancelButtonsOn: html
	html fieldSet
		jqmContainerTwoColumn;
		with: [
			html div
				jqmColumnOne;
				with: [
					html button
						with: 'Cancel' ].
			html div
				jqmColumnTwo;
				with: [			
					html button
						with: 'Submit' ] ]! !

WebComponent subclass: #WebTemperature
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebTemperature methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 08:25'!
renderContentOn: html

self renderDefaultHeaderOn: html.
html div
      jqmContent;
      with: [
         	html heading
		level2;
		with: 'Temperature Values'.].

self renderTemperatureListOn: html inset: true  anchor: false. 	
      
self renderDefaultFooterOn: html! !

!WebTemperature methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 08:26'!
renderTemperatureListOn: html inset: inset anchor: anchor
	| list |
		
	list := html unorderedList
		jqmListView;
		class: 'temperature'.
	
	inset ifTrue: [ list jqmEnableInset ].
	list with: [
		self temperatureSensors  do: [ :each |
			html listItem: [
				| renderer |
				renderer := [
					html render: each.
					html text: ' '.
					html span
						class: 'ui-li-count';
						with: 50 atRandom ].
				anchor
					ifTrue: [ html anchor: renderer ]
					ifFalse: [ html render: renderer ] ] ] ]! !

!WebTemperature methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 01:27'!
style
	^ '
		.temperature {
			width: 80%;
			margin: auto;
			font-family: verdana;
			color: #3e3e3e;
			font-size:13px;
		}
		
		.ui-li-count {

		}'! !

!WebTemperature methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/10/2013 08:25'!
temperatureSensors
	^ #(Sensor1 Sensor2 Sensor3)! !

WAComponent subclass: #WebHeader
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebHeader methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/4/2013 01:48'!
renderContentOn: html

	html div
		id: 'header'; 
		with: [html heading: 'Home Automation System']! !

!WebHeader methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 14:57'!
style
	^ '
		#header {
			background: #CCCCCC;
		}'! !

WAComponent subclass: #WebLineChart
	instanceVariableNames: 'xdata ydata title units'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!WebLineChart methodsFor: 'rendering' stamp: 'IsraSunhachawi 7/12/2013 01:42'!
renderContentOn: html
	html div
		script:
			(html
				highchart: [ :chart | 
					chart chart defaultSeriesType: 'line'.
					chart title text: title.
					chart xAxis categories: xdata.
					chart yAxis title text: units.
					chart
						series:
							(Array
								with:
									(chart step
										name: 'Measured';
										data: ydata)) ])! !

!WebLineChart methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/5/2013 23:53'!
xData: xDataCollection yData: yDataCollection

super initialize.
xdata:= xDataCollection.
ydata:= yDataCollection.! !

!WebLineChart methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/12/2013 01:41'!
xData: xDataCollection yData: yDataCollection title: aString units: aUnit

super initialize.
xdata:= xDataCollection.
ydata:= yDataCollection.
title := aString.
units := aUnit.! !

TestCase subclass: #DeviceTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceTest methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 10:05'!
testXml
	|xml dev stream xmlString doc device|
	xmlString := '<Device><name>arduino</name><ip>10.9.13.11</ip></Device>'.
	stream := xmlString readStream.
	doc := XMLDOMParser parseDocumentFrom: stream.
	device := Device new.
	device name: (doc findElementNamed: 'name') contentString.
	device ip: (doc findElementNamed: 'ip') contentString.
	
	Transcript show: device asXml.! !

TestCase subclass: #NotificationManagerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!NotificationManagerTest methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:14'!
testNotificationMessages
	|alarm device deviceSensor|
	NotificationManager reset.
	
	device := Device new.
	deviceSensor := TemperatureSensor new.
	device name: 'Gateway'.
	device addSensor: deviceSensor.
	alarm := Alarm setAlarm: 'temperature' device:device deviceSensor: deviceSensor lower: 60 upper: 100.
	
	self assert: (NotificationManager singleton messages size = 0).
	deviceSensor updateDataNow: 104.
	
	self assert: (NotificationManager singleton messages size = 1).! !

TestCase subclass: #StateMachineTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!StateMachineTest methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/14/2013 00:17'!
testSimpleStateMachine
	|sm aFireCondition aResetCondition event observer |
	aFireCondition := ThresholdCondition new low:70 high:100.
	"aResetCondition := InverseCondition new condition: aFireCondition."
	
	"A condition that is always false will disable automatic reset of an alarm"
	aResetCondition := FalseCondition new.
	
	sm:= StateMachine newWithName: 'myFirstSM' fireCondition: aFireCondition  resetCondition: aResetCondition.
	observer := TestAlarmObserver new.
	sm attachObserver: observer.
	
	event := LightSensor new.
	event updateDataNow: 4.
	sm handle: event.
	
	self assert: observer firedCount = 1.
	self assert: observer resetCount = 0.
	
	
	! !

TestCase subclass: #ValueWithTimeStampTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!ValueWithTimeStampTest methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:53'!
test
	| history t1 t2 t3 t4 |
	t1 := DateAndTime yesterday.
	t2 := DateAndTime today.
	t3 := DateAndTime now.
	t4 := DateAndTime tomorrow.
	(history := ValueWithTimeStamp new) at: t1 become: 1.
	history at: t4 become: 4.
	history at: t3 become: 3.
	history at: t2 become: 2.
	
	self assert: (history valueAt: t1) = 1.
	self assert: (history valueAt: t2) = 2.
	self assert: (history valueAt: t3) = 3.
	self assert: (history valueAt: t4) = 4.
	
	Transcript show: history.
	Transcript show: (history timeStampAndValue).
	Transcript show: 'testEnd'; cr.! !

Object subclass: #Alarm
	instanceVariableNames: 'lowerThreshold upperThreshold name device deviceSensor alarmListener'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!Alarm class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 21:04'!
setAlarm: aName device: aDevice deviceSensor: aDeviceSensor lower: aNumber1 upper: aNumber2

| alarm |
alarm := (Alarm new) setAlarm: aName device: aDevice deviceSensor: aDeviceSensor lower: aNumber1 upper: aNumber2.
^ alarm ! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 20:59'!
device
^ device! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 20:18'!
deviceSensor
^ deviceSensor! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 19:18'!
initialize
	super initialize.
! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 11:20'!
lower

^ lowerThreshold! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 15:27'!
name

^name! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:08'!
setAlarm: aName device: aDevice deviceSensor: aDeviceSensor lower: aNumber1 upper: aNumber2
| aFireCondition aResetCondition sm |
super initialize.
name := aName.

alarmListener := AlarmObserver new.

device := aDevice.
deviceSensor := aDeviceSensor.
lowerThreshold := aNumber1.
upperThreshold := aNumber2.

aFireCondition := ThresholdCondition new low: aNumber1 high: aNumber2.
aResetCondition := InverseCondition new condition: aFireCondition.
sm := StateMachine newWithName: aName fireCondition: aFireCondition resetCondition: aResetCondition.
alarmListener setStateMachine: sm.
sm attachObserver: alarmListener.
sm attachObserver: (AlarmNotificationListener newWithAlarm: self).
deviceSensor attachObserver: alarmListener.

! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 19:35'!
state

^ alarmListener state! !

!Alarm methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 11:31'!
upper

^ upperThreshold! !

Object subclass: #AlarmManager
	instanceVariableNames: 'alarms mutex'
	classVariableNames: 'uniqueInstance'
	poolDictionaries: ''
	category: 'Sensors'!

!AlarmManager class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:22'!
getAlarm: aAlarmName
	^AlarmManager singleton getAlarm: aAlarmName.! !

!AlarmManager class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:21'!
reset
	uniqueInstance := nil.! !

!AlarmManager class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:21'!
singleton
	uniqueInstance ifNil: [uniqueInstance := self basicNew initialize].
       ^uniqueInstance! !

!AlarmManager methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:25'!
addAlarm: alarm
	mutex critical: [ alarms at: (alarm name) put: alarm ].
	! !

!AlarmManager methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:25'!
addDevice: device
	mutex critical: [ alarms at: (device name) put: device ].
	! !

!AlarmManager methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:23'!
alarms
	^alarms! !

!AlarmManager methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:26'!
getAlarm: alarmName
	^alarms at: alarmName.! !

!AlarmManager methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/13/2013 10:24'!
initialize
	super initialize.
	alarms := Dictionary new.
	mutex := Mutex new.! !

Object subclass: #Condition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!Condition methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:36'!
isMet: anEvent
	self subclassResponsibility.! !

Condition subclass: #FalseCondition
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!FalseCondition methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/14/2013 00:16'!
isMet: anEvent
	^ false! !

Condition subclass: #InverseCondition
	instanceVariableNames: 'condition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!InverseCondition methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:40'!
condition: aCondition
	condition := aCondition.
! !

!InverseCondition methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:39'!
isMet: anEvent
	^ (condition isMet: anEvent) not.! !

Condition subclass: #ThresholdCondition
	instanceVariableNames: 'low high'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!ThresholdCondition methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:38'!
isMet: anEvent
	|value|
	value := anEvent value.
	^(value < low) | (value>high).! !

!ThresholdCondition methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:39'!
low: lowVal high: highVal
	low := lowVal.
	high := highVal.! !

Object subclass: #DScheduleItem
	instanceVariableNames: 'type name isEnabled lastTrigeredTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

DScheduleItem subclass: #CountdownItem
	instanceVariableNames: 'setOffTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!CountdownItem class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/18/2013 23:57'!
name: aName seconds: numOfSeconds
	^(CountdownItem new name: aName; setTimerSeconds: numOfSeconds; enable)
	! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:56'!
initialize
	super initialize.! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:01'!
secondsToSetOff
	 ^ (setOffTime asSeconds) - (DateAndTime now asSeconds)! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:52'!
setDefaultName
	name := 'a CountdownItem'! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:57'!
setOffTime
	^setOffTime.! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:58'!
setOffTime: aDateAndTime
	setOffTime := aDateAndTime.! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:02'!
setTimerSeconds: numOfSeconds
	setOffTime := DateAndTime fromSeconds: (DateAndTime now asSeconds + numOfSeconds)! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:52'!
setType
	type := #CountdownItem.! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 21:57'!
shouldTriggerNow
	"ask the object to test whether it is time to trigger according to the schedule"
	(isEnabled) ifFalse: [ ^false ].

	^(self secondsToSetOff <= 0 )! !

!CountdownItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:23'!
trigger
	"take action"
	Transcript cr; show: 'aCountdownItem triggered!!'.
	lastTrigeredTime := DateAndTime now asSeconds.
	
	"disable (turn off) the this schedule item"
	self disable.! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:11'!
disable
	isEnabled := false ! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:10'!
enable
	isEnabled := true. ! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:14'!
initialize
	super initialize.
	isEnabled := false.
	lastTrigeredTime := -1. "-1 means never triggered."
	self setType.
	self setDefaultName.! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:10'!
isEnabled
	^isEnabled! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:40'!
name
	^name! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:40'!
name: aName
	name := aName! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:40'!
setDefaultName
	^self subclassResponsibility! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:40'!
setType
	^self subclassResponsibility! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:16'!
shouldTriggerNow
	"ask the object to test whether it is time to trigger according to the schedule"
	^self subclassResponsibility! !

!DScheduleItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 20:20'!
trigger
	"take action"
	^self subclassResponsibility! !

DScheduleItem subclass: #DailyItem
	instanceVariableNames: 'dayOfWeek triggerTime'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DailyItem class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/19/2013 00:05'!
name: aName dayOfWeek: anInteger time: aTime
	^(DailyItem new name: aName; dayOfWeek: anInteger; triggerTime: aTime; enable)! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/18/2013 21:55'!
dayOfWeek
	^dayOfWeek! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/18/2013 21:56'!
dayOfWeek: anInteger
	"dayOfWeek  1: AllDays.  2: Weekdays only..  3: Weekends only"
	dayOfWeek := anInteger! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/19/2013 00:15'!
hasBeenTriggeredToday
	|lastDate|
	lastDate := (DateAndTime fromSeconds: lastTrigeredTime) asDate.
	
	(lastDate = Date today) ifTrue: [^true] ifFalse: [^false].! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 21:47'!
initialize
	super initialize.
	"dayOfWeek  1: AllDays.  2: Weekdays only..  3: Weekends only"
	dayOfWeek := 1.! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:50'!
setDefaultName
	name := 'a DailyItem'! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:51'!
setType
	type := #DailyItem.! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/19/2013 00:14'!
shouldTriggerNow
	"ask the object to test whether it is time to trigger according to the schedule"
	|today now|

	(self hasBeenTriggeredToday) ifTrue: [^false]. "if has been triggered today"

	(isEnabled) ifFalse: [ ^false ].
	
	(dayOfWeek = 0) ifTrue: [^false]. "Not setup yet"
	
	today := DateAndTime now dayOfWeek. "1: sunday, 2: monday. etc"
	now := Time now.
	
	"test weekdays only"
	(dayOfWeek = 2 and: (today = 1  or: today = 7))  ifTrue: [^false].
	
	"test weekends only"
	(dayOfWeek = 3 and: (today >= 2  and: today <=6 ))  ifTrue: [^false].
	
	(now > triggerTime) ifTrue: [^true] ifFalse: [^false].
! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/19/2013 00:02'!
trigger
	"take action"
	Transcript cr; show: 'aDailyItem triggered!!'.
	lastTrigeredTime := DateAndTime now asSeconds.! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/18/2013 23:29'!
triggerTime
	^triggerTime! !

!DailyItem methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/18/2013 23:30'!
triggerTime: aTime
	triggerTime := aTime! !

Object subclass: #DScheduler
	instanceVariableNames: 'items interval daemonProcess daemonOn mutex'
	classVariableNames: 'uniqueInstance'
	poolDictionaries: ''
	category: 'Sensors'!

!DScheduler class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:25'!
reset
	uniqueInstance ifNotNil: [uniqueInstance stopDaemon].
	uniqueInstance := nil.! !

!DScheduler class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:04'!
singleton
	uniqueInstance ifNil: [uniqueInstance := self basicNew initialize].
       ^uniqueInstance! !

!DScheduler methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/20/2013 12:50'!
addItem: aDScheduleItem
	items at: (aDScheduleItem name) put: aDScheduleItem

	! !

!DScheduler methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/20/2013 12:48'!
daemonRoutine
	[daemonOn] whileTrue: 
	[
		Transcript show: 'daemonProcess is working'; cr. 
		
		items valuesDo: [ :item | (item shouldTriggerNow) ifTrue: [item trigger] ].
	
		(Delay forSeconds: interval) wait
	].
	! !

!DScheduler methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/20/2013 12:43'!
getItemNamed: aNameString
	^ items at: aNameString

	! !

!DScheduler methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:18'!
initialize
	super initialize.
	items := Dictionary new.
	daemonProcess := nil.
	daemonOn := false. "daemon is not on"
	interval := 5. "5 seconds"
	mutex := Mutex new.! !

!DScheduler methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/20/2013 12:43'!
removeItemNamed: aNameString
	items removeKey: aNameString

	! !

!DScheduler methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:18'!
startDaemon
	daemonOn := true.
	daemonProcess := [self daemonRoutine] forkAt: 50.

	! !

!DScheduler methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/17/2013 19:28'!
stopDaemon
	daemonOn ifFalse: [
		Transcript show: 'DScheduler had been stopped.'; cr.
		^nil].
	
	daemonOn := false.
	[daemonProcess isTerminated] whileFalse: [
		Transcript show: 'waiting DScheduler daemon to stop...'; cr.
		(Delay forSeconds: 1) wait].
	
	daemonProcess := nil.
	
	Transcript show: 'DScheduler stopped.'; cr.

	! !

Object subclass: #Device
	instanceVariableNames: 'name ip sensors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!Device class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 21:18'!
newFromXml: xmlString
	|device aName anIp doc stream aSensor|
	device := Device new.
	stream := xmlString readStream.
	doc := XMLDOMParser parseDocumentFrom: stream.
	device name: (doc findElementNamed: 'name') contentString.
	device ip: (doc findElementNamed: 'ip') contentString.
	(doc allElementsNamed: 'sensor') do:
		[ :each | aSensor := DeviceSensor newFromXml: each.
				aSensor ifNotNil: [device addSensor: aSensor] ].
	^device.! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 10:03'!
addSensor: aDeviceSensor
	|d|
	sensors at: (aDeviceSensor name) put: aDeviceSensor.
	d := (aDeviceSensor isKindOf: Control).
	(aDeviceSensor isKindOf: Control) ifTrue:[ aDeviceSensor transport: (HTTPTransport forDevice: self)].
! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 21:02'!
asXml
	|stream|
	stream := WriteStream on: (String new: 1).
	stream nextPutAll: '<Device>'.
	stream nextPutAll: '<name>'.
	stream nextPutAll: name.
	stream nextPutAll: '</name>'.
	
	stream nextPutAll: '<ip>'.
	stream nextPutAll: ip.
	stream nextPutAll: '</ip>'.
	
	sensors keysAndValuesDo: [ :key :sensor | 
		stream nextPutAll: '<sensor>'.
		stream nextPutAll: '<type>'.
		stream nextPutAll: sensor type.
		stream nextPutAll: '</type>'.
		stream nextPutAll: '<name>'.
		stream nextPutAll: key.
		stream nextPutAll: '</name>'.
		stream nextPutAll: '</sensor>'.].
	
	stream nextPutAll: '</Device>'.
	^stream contents.! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 20:32'!
getSensorByName: aSensorName
	^ sensors at: aSensorName.! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 21:07'!
initialize
	super initialize.
	sensors := Dictionary new.
	name := ''.
	ip := ''.
! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 00:40'!
ip
	^ip! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 00:39'!
ip: anIp
	ip := anIp! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:29'!
name
	^name! !

!Device methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:29'!
name: aName
	name := aName! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 21:16'!
sensors
	^sensors! !

!Device methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 21:17'!
sensors: aDict
	sensors := aDict! !

Object subclass: #DeviceManager
	instanceVariableNames: 'devices mutex'
	classVariableNames: 'uniqueInstance'
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceManager class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/9/2013 22:44'!
getDevice: aDeviceName
	^DeviceManager singleton getDevice: aDeviceName.! !

!DeviceManager class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:16'!
reset
	uniqueInstance := nil.! !

!DeviceManager class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:27'!
singleton
	uniqueInstance ifNil: [uniqueInstance := self basicNew initialize].
       ^uniqueInstance! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:36'!
addDevice: device
	mutex critical: [ devices at: (device name) put: device ].
	! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 6/30/2013 23:29'!
devices
	^devices! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/1/2013 10:30'!
getDevice: deviceName
	^devices at: deviceName.! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:35'!
initialize
	super initialize.
	devices := Dictionary new.
	mutex := Mutex new.! !

!DeviceManager methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 18:57'!
removeDevice: deviceName
	mutex critical: [ devices removeKey: deviceName. ]! !

Object subclass: #DeviceSensorFactory
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!DeviceSensorFactory class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:18'!
createSensor: aString
	aString = 'LightSensor' ifTrue: [^ LightSensor new].
	aString = 'TemperatureSensor' ifTrue: [^ TemperatureSensor new].
	aString = 'LightControl' ifTrue: [^ LightControl new].
	^nil! !

Object subclass: #HTTPTransport
	instanceVariableNames: 'device'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

HTTPTransport class
	instanceVariableNames: 'device'!

HTTPTransport class
	instanceVariableNames: 'device'!

!HTTPTransport class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:34'!
forDevice: aDevice
	^HTTPTransport new device: aDevice.! !

!HTTPTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 09:34'!
device: aDevice
	device := aDevice.! !

!HTTPTransport methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/19/2013 02:03'!
set: value for: name
	|url deviceIp|
	deviceIp := device ip.
	 url:= 'http://', deviceIp , '/command/', name, '/', (value asString).
	Transcript show: url.
	 ZnClient new post: url contents: ''.
! !

Object subclass: #NotificationManager
	instanceVariableNames: 'messages'
	classVariableNames: 'uniqueInstance'
	poolDictionaries: ''
	category: 'Sensors'!

!NotificationManager class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:13'!
reset
	NotificationManager singleton reset.! !

!NotificationManager class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 23:10'!
singleton
	uniqueInstance ifNil: [uniqueInstance := self basicNew initialize].
       ^uniqueInstance! !

!NotificationManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 23:27'!
initialize
	super initialize.
	messages := OrderedCollection new.! !

!NotificationManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 23:26'!
messages
	^messages.! !

!NotificationManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:02'!
notifyChanges: message
	Transcript show: message.
	messages add: message.! !

!NotificationManager methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:13'!
reset
	messages := OrderedCollection new.! !

Object subclass: #Observable
	instanceVariableNames: 'observers'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

Observable subclass: #DeviceSensor
	instanceVariableNames: 'type name data dataUnit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

DeviceSensor subclass: #Control
	instanceVariableNames: 'state transport'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!Control methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:19'!
off
	state := 'off'.! !

!Control methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:19'!
on
	state := 'on'.! !

!Control methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/19/2013 01:59'!
sendToDevice: value
	transport set: value for: name.! !

!Control methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:19'!
setUnit
	dataUnit := #OnOff! !

!Control methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 00:07'!
transport: aTransport
	transport := aTransport.! !

!Control methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:20'!
value
	 ^state! !

!Control methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 00:04'!
value: aValue
	 state:= aValue.
	self sendToDevice: aValue.! !

Control subclass: #LightControl
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!LightControl methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 00:15'!
setDefaultName
	name := #aLightControl! !

!LightControl methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/10/2013 17:20'!
setType
	type := #LightControl! !

!DeviceSensor class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 21:31'!
newFromXml: xmlElement
	|type aName doc aSensor|
	type := (xmlElement findElementNamed: 'type') contentString.
	aName := (xmlElement findElementNamed: 'name') contentString.
	aSensor := DeviceSensorFactory createSensor: type.
	aSensor ifNotNil: [aSensor name: aName].
	^aSensor.! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 22:05'!
asDataXml
	|stream|
	stream := WriteStream on: (String new: 1).
	stream nextPutAll: '<data>'.
	
	stream nextPutAll: (self value asString).
	
	stream nextPutAll: '</data>'.
	
^ stream contents.! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 00:01'!
asXml
	|stream|
	stream := WriteStream on: (String new: 1).
	stream nextPutAll: '<Sensor>'.
	stream nextPutAll: '<data>'.
	
	stream nextPutAll: (self value asString).
	
	stream nextPutAll: '</data>'.
	stream nextPutAll: '</Sensor>'.
	
^ stream contents.! !

!DeviceSensor methodsFor: 'accessing' stamp: 'WeiqiLuo 7/6/2013 12:29'!
data
	^data
! !

!DeviceSensor methodsFor: 'initialization' stamp: 'WeiqiLuo 7/11/2013 20:39'!
initialize
	super initialize.
	self setType.
	self setUnit.
	self setDefaultName.
	data := ValueWithTimeStamp new.! !

!DeviceSensor methodsFor: 'accessing' stamp: 'DanishShrestha 7/5/2013 14:10'!
name
	^name! !

!DeviceSensor methodsFor: 'accessing' stamp: 'DanishShrestha 7/5/2013 14:11'!
name: aName
	name := aName! !

!DeviceSensor methodsFor: 'printing' stamp: 'WeiqiLuo 7/6/2013 12:37'!
printOn: aStream
	aStream nextPutAll: 'Sensor type: '; nextPutAll: type; cr.
	aStream nextPutAll: 'Sensor data: (in unit of: '; nextPutAll: dataUnit ; nextPut: $) ;cr.
	data printOn: aStream.! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 20:39'!
setDefaultName
	^self subclassResponsibility! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:39'!
setType
	^self subclassResponsibility! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:30'!
setUnit
	^self subclassResponsibility! !

!DeviceSensor methodsFor: 'accessing' stamp: 'WeiqiLuo 7/5/2013 21:00'!
type
	^type
! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:24'!
updateData: anAmount at: aDateAndTime
	data at: aDateAndTime become: anAmount.
! !

!DeviceSensor methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:18'!
updateDataNow: anAmount
	|aDateAndTime|
	aDateAndTime := DateAndTime now.
	self updateData: anAmount at: aDateAndTime.
	self notifyChanges: self
! !

!DeviceSensor methodsFor: 'evaluating' stamp: 'DanishShrestha 7/12/2013 02:13'!
value
	 ^data valueAt: (DateAndTime now).! !

!DeviceSensor methodsFor: 'vector functions' stamp: 'DanishShrestha 7/5/2013 14:10'!
valueAt: channel
	self subclassResponsibility.! !

DeviceSensor subclass: #LightSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!LightSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:42'!
initialize
	super initialize.! !

!LightSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 20:40'!
setDefaultName
	name := #aLightSensor! !

!LightSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:43'!
setType
	type := #LightSensor! !

!LightSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:32'!
setUnit
	dataUnit := #Luminance! !

!LightSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:18'!
unit
	^ 'Lumen'! !

DeviceSensor subclass: #TemperatureSensor
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:43'!
initialize
	super initialize.! !

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/11/2013 20:40'!
setDefaultName
	name := #aTemperatureSensor! !

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 20:44'!
setType
	type := #TemperatureSensor! !

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 12:33'!
setUnit
	dataUnit := #Fahrenheit! !

!TemperatureSensor methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/5/2013 14:19'!
unit
	^ 'F'! !

!Observable methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:58'!
attachObserver: observer
	observers add: observer.! !

!Observable methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:00'!
initialize
	super initialize.
	observers := OrderedCollection new.! !

!Observable methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 08:21'!
notifyChanges: change
	observers do:[:each | each notify: change]! !

!Observable methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:59'!
removeObserver: observer
	observers delete: observer.! !

Observable subclass: #StateMachine
	instanceVariableNames: 'state resetState firedState name listeners'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!StateMachine class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:52'!
newWithName: aName fireCondition: aFireCondition resetCondition:aResetCondition
	^StateMachine new name: aName fireCondition: aFireCondition resetCondition:aResetCondition! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:46'!
handle: anEvent
	state handle: anEvent context: self.! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:54'!
initialize
	super initialize.
	self transitionToReset.
	listeners := OrderedCollection new.! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:48'!
name: aName fireCondition: aFireCondition resetCondition:aResetCondition
	name := aName.
	firedState := FiredState new condition: aResetCondition.
	resetState := ResetState new condition: aFireCondition.
	self transitionToReset.! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 08:23'!
notifyStateChanges: change
	observers do:[:each | each notifyStateChange: change]! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:50'!
state

^state! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 08:23'!
transitionToFired
	state := firedState.
	self notifyStateChanges: state.! !

!StateMachine methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 08:23'!
transitionToReset
	state := resetState.
	self notifyStateChanges: state.! !

Object subclass: #Observer
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

Observer subclass: #AlarmNotificationListener
	instanceVariableNames: 'alarm'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

AlarmNotificationListener class
	instanceVariableNames: 'alarm'!

AlarmNotificationListener class
	instanceVariableNames: 'alarm'!

!AlarmNotificationListener class methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 23:20'!
newWithAlarm: anAlarm
	^AlarmNotificationListener new alarm: anAlarm.
! !

!AlarmNotificationListener methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 23:20'!
alarm: anAlarm
	alarm := anAlarm.! !

!AlarmNotificationListener methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 23:28'!
notify: event
	|message|
	(event isKindOf: FiredState) ifTrue: [message := 'Alarm ', (alarm name), ' Fired at ', (DateAndTime new)].
	(event isKindOf: ResetState) ifTrue: [message := 'Alarm ', (alarm name), ' Reset at ', (DateAndTime new)].
	NotificationManager singleton notifyChanges: message! !

!AlarmNotificationListener methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/18/2013 19:12'!
notifyStateChange: event
	|message|
	(event isKindOf: FiredState) ifTrue: [message := 'Alarm ', (alarm name), ' Fired at ', (DateAndTime new printString)].
	(event isKindOf: ResetState) ifTrue: [message := 'Alarm ', (alarm name), ' Reset at ', (DateAndTime new printString)].
	NotificationManager singleton notifyChanges: message! !

Observer subclass: #AlarmObserver
	instanceVariableNames: 'firedCount resetCount stateMachine'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:14'!
firedCount
	^firedCount! !

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:24'!
initialize
	super initialize.
	firedCount := 0.
	resetCount := 0.
	stateMachine := StateMachine new.! !

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 08:26'!
notify: event

	| currentState |
	stateMachine handle: event.
	currentState := stateMachine state.
	Transcript show: 'Sensor value updated to: ';
				show: event value; cr.
	Transcript show: 'Current state: '.
	Transcript show: stateMachine state; cr.
				
	
	"(currentState class == FiredState) ifTrue: [
		firedCount := firedCount + 1.
	].

	(currentState class == ResetState) ifTrue: [
		resetCount := resetCount + 1.
	]."! !

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/17/2013 23:18'!
notifyStateChange: event

	Transcript show: 'State change notification: '.
	Transcript show: event; cr.
	
	(event class == FiredState) ifTrue: [
		firedCount := firedCount + 1.
	].

	(event class == ResetState) ifTrue: [
		resetCount := resetCount + 1.
	].
! !

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:14'!
resetCount
	^resetCount! !

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:13'!
setStateMachine: aStateMachine

stateMachine := aStateMachine! !

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:31'!
state

^ stateMachine state.! !

!AlarmObserver methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:30'!
stateMachine

^stateMachine! !

!Observer methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 13:29'!
notify: change
	self subclassResponsibility ! !

Observer subclass: #TestAlarmObserver
	instanceVariableNames: 'firedCount resetCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!TestAlarmObserver methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 20:33'!
firedCount
	^firedCount! !

!TestAlarmObserver methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 20:30'!
initialize
	super initialize.
	firedCount := 0.
	resetCount := 0.
	! !

!TestAlarmObserver methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 20:33'!
notify: event
	(event class == FiredState) ifTrue: [
		firedCount := firedCount + 1.
	].

	(event class == ResetState) ifTrue: [
		resetCount := resetCount + 1.
	].! !

!TestAlarmObserver methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 20:34'!
resetCount
	^resetCount! !

Object subclass: #RawData
	instanceVariableNames: 'timestamp measurement'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!RawData commentStamp: 'IsraSunhachawi 7/3/2013 01:33' prior: 0!
RawData is used to store measurements provided by sensors.

Instance Variables:
	deviceId	<ProtoObject | PseudoContext>
	timestamp	<ProtoObject | PseudoContext>
	measurement	<ProtoObject | PseudoContext>!

!RawData class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:40'!
createSampleDatabase
    Database := OrderedCollection new
        add: (self name: 'Bob Jones' emailAddress: 'bob@nowhere.com');
        add: (self name: 'Steve Smith' emailAddress: 'sm@somewhere.com');
        yourself! !

!RawData class methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:39'!
device: deviceString timestamp: aTime measurement: aMeasurement
    ^ self new
          deviceId: deviceString;
          timestamp: aTime;
	    measurement: aMeasurement;
          yourself! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:36'!
measurement

	^ measurement! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:36'!
measurement: aNumber

	measurement := aNumber! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/6/2013 00:41'!
time: aTimeStamp measurement: aMeasurement

timestamp := aTimeStamp.
measurement := aMeasurement.! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:35'!
timeStamp: aDate

	timestamp := aDate! !

!RawData methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/3/2013 01:35'!
timestamp

	^ timestamp! !

Object subclass: #State
	instanceVariableNames: 'condition'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

State subclass: #FiredState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!FiredState methodsFor: 'as yet unclassified' stamp: 'IsraSunhachawi 7/17/2013 07:49'!
handle: anEvent context: aContext
	(condition isMet: anEvent) ifTrue:[aContext transitionToReset.].! !

State subclass: #ResetState
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Sensors'!

!ResetState methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 20:28'!
handle: anEvent context: aContext
	(condition isMet: anEvent)  ifTrue:[aContext transitionToFired.].! !

!State methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:45'!
condition: aCondition
	condition := aCondition.
! !

!State methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:46'!
handle: anEvent context: aContext
	self subclassResponsibility.! !

!State methodsFor: 'as yet unclassified' stamp: 'DanishShrestha 7/11/2013 12:44'!
isMet: anEvent
	^condition isMet: anEvent! !

Object subclass: #ValueWithTimeStamp
	instanceVariableNames: 'second value'
	classVariableNames: 'Epoch'
	poolDictionaries: ''
	category: 'Sensors'!

!ValueWithTimeStamp class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:42'!
initialize
	"self initialize"
	Epoch := DateAndTime fromSeconds: 0.! !

!ValueWithTimeStamp class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:42'!
new
	^ super new initialize
! !

!ValueWithTimeStamp class methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:59'!
zero
	^ self new at: 0 become: 0
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:13'!
at: aDate at: aTime become: anAmount 
	| index aDateAndTime aSecond|
	aDateAndTime := DateAndTime year: aDate year day: aDate day hour: aTime hour minute: aTime minute second: aTime second.
	self at: aDateAndTime become: anAmount.
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:13'!
at: aDateAndTime become: anAmount 
	| index aSecond|
	aSecond := aDateAndTime asSeconds.
	self atSecond: aSecond become: anAmount.
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:02'!
atSecond: aSecond become: anAmount 
	| index |
	second add: aSecond.
	index := second indexOfStartOfIntervalContaining: aSecond.
	value add: anAmount beforeIndex: index.! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:55'!
indexForAccessing: aSecond 
	"Create index of slot for aSecond, creating it if necessary."
	| index |
	(index := second indexOfStartOfIntervalContaining: aSecond) = 0 ifTrue: 
		[second add: aSecond.
		value addFirst: 0.
		^ 1].
	(second at: index) = aSecond ifTrue: [^index].
	second add: aSecond.
	value add: (value at: index) beforeIndex: index + 1.
	^ index + 1! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 10:52'!
initialize
	second := SortedSequence new.
	value := OrderedCollection new.
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:48'!
printOn: aStream 
	second with: value do: 
		[: nextSecond : nextValue | 
		aStream 
			nextPut: $(;
			print: (DateAndTime fromSeconds: nextSecond);
			nextPutAll: ' -> ';
			print: nextValue;
			nextPut: $);
			cr].! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/5/2013 21:54'!
second
	^second! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:52'!
timeStampAndValue
	|aDictionary aDateAndTime|
	aDictionary := Dictionary new.
	second with: value do: 
		[: nextSecond : nextValue | 
		aDateAndTime := DateAndTime fromSeconds: nextSecond.
		aDictionary at: aDateAndTime put: nextValue
		].
	^aDictionary! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 10:58'!
value
	^value! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:24'!
valueAt: aDateAndTime
	"Return value at aDateAndTime"
	| index aSecond|
	aSecond := aDateAndTime asSeconds.
	^ self valueAtSecond: aSecond
! !

!ValueWithTimeStamp methodsFor: 'as yet unclassified' stamp: 'WeiqiLuo 7/6/2013 11:23'!
valueAtSecond: aSecond 
	"Return value at aSecond"
	| index |
	index := second indexOfStartOfIntervalContaining: aSecond.
	index = 0 ifTrue: [self error: 'time is too early'].
	^value at: index! !
WebHome initialize!
ValueWithTimeStamp initialize!
